/* XmGoBang: (c) 1995-2008 by Andreas S. Krebs */

/* the needed include files */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <limits.h>
#include <Xm/Xm.h>

#include <X11/Shell.h>
#include <X11/Composite.h>
#include <X11/xpm.h>

#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/Protocols.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "config.h"


/* my defines */

#define	APPL_CLASS	"XmGoBang"

#define	GOOD	0
#define BAD	1

/* data about the images */
#define FIELD_SIZE	23
#define IMAGE_SIZE	17
#define BORDER_SIZE	6
#define IMAGE_OFFSET	((FIELD_SIZE-IMAGE_SIZE)/2)
#define FIELD_OFFSET  	(IMAGE_SIZE/2 + BORDER_SIZE +1)	

/* the possible values a field can have */
#define FIELD_EMPTY	0
#define FIELD_PLAYER1	1
#define FIELD_PLAYER2	2
#define FIELD_BORDER	3

/* id's for the cheat widgets */
#define UNDO_WIDGET	0
#define REDO_WIDGET	1
#define HINT_WIDGET	2

/* the possible player types */
#define	PLAYER_COMPUTER	0
#define PLAYER_HUMAN	1

#define PLAYER_HINT	2	/* no real player, only for hints */


#define PLAYER1_COMPUTER	PLAYER_COMPUTER
#define PLAYER1_HUMAN		PLAYER_HUMAN
#define PLAYER2_COMPUTER	(PLAYER_COMPUTER+0x10)
#define PLAYER2_HUMAN		(PLAYER_HUMAN+0x10)

/* game status */
#define GAME_NEW	0
#define GAME_RUNNING	1
#define GAME_PAUSED	2
#define GAME_FINISHED	3

/* the 4 possible directions */
#define DIR_HORIZ	0
#define DIR_LEFTUP	1
#define DIR_VERT	2
#define DIR_RIGHTUP	3

/* play modes */
#define	PLAY_NORMAL	0
#define PLAY_DEFENSIVE	1
#define PLAY_OFFENSIVE	2

/* cheat modes */
#define CHEAT_NONE	0
#define CHEAT_HINT	1
#define CHEAT_UNDO	2

/* my structs */

typedef struct  
{
  Cardinal 	Type;		/* human or computer ? */
  Pixel		TopColor;	/* topShadow for player stone */
  Pixel		Color;		/* color of player and stone */
  Pixel		BottomColor;	/* bottomShadow for player stone */
  String	Name;		/* name of player */
  long		Seconds;	/* time played by player */
} GoPlayer;

typedef struct
{
  GoPlayer	Player[2];	/* 2 players */
  short		PlayersTurn;	/* whose turn ? (player 0 or 1) */
  short		StartPlayer;	/* who starts ? */
  Pixel 	BoardTopColor;		/* topShadow for board grid */
  Pixel		BoardColor;		/* color for board grid */
  Pixel		BoardBottomColor;	/* bottomShadow for board grid */
  Pixel		BoardBackground;	/* color for board background */
  Pixel		BoardBorderColor;	/* color for board border */
  Pixel		HintTopColor;		/* topcolor for hint stone */
  Pixel		HintColor;		/* color for hint stone */
  Pixel		HintBottomColor;	/* bottomShadow for hint stone */
  short 	GameStatus;	/* Status of Game: NEW, PAUSED, RUNNING, FINISHED */ 
  short		Columns, Rows;	/* size of board in Columns and Rows */
  int		Width, Height;	/* size of board in Pixels */
  short		PlayMode;	/* play offensive, defensive or normal */
  short		CheatMode;	/* none, hint, undo possible */
  Boolean	SoundOn;	/* play sounds or not */
} GoGame;
  
typedef struct
{
  char *BoardField;		/* Memory with Rows * Columns fields with status */
  char *MovesDone;		/* Stack containing all done moves */
  int MoveNumber;		/* Number of the current move */ 
  int LastMoveDone;		/* Copy of the highest MoveNumber (for Redo) */
  int NumberOfFields;		/* = Rows * Columns */
} GoBoard;

typedef struct
{
  short SameColor;	/* number of chips with same color */
  short SizeOfArea;	/* number of free fields in direction */
  short OpenEnds;	/* number of open ends */
  short SameInArea;	/* chips with same color, but 1 free in between */   
} DirInfo;

/* global prototypes */

extern Widget initMenuWidget(Widget);			/* widgets.c */
extern Widget initFormWidget(Widget);			/* widgets.c */
extern void exitQuestionCB(Widget, XtPointer, XtPointer);	/* widgets.c */
extern void popdownCB(Widget, XtPointer, XtPointer);	/* widgets.c */
extern void setPlayersTurn(short);			/* widgets.c */
extern void startPlayerTimer(short);			/* widgets.c */
extern void switchOffWidget(short);			/* widgets.c */
extern void switchOnWidget(short);			/* widgets.c */
extern void setGameStatus(short);			/* widgets.c */
extern void setPlayerTimer(short);			/* widgets.c */


extern void getApplResources(Widget);			/* resources.c */


extern void exitCB(Widget, XtPointer, XtPointer);	/* xmgobang.c */
extern void setCloseCallback(Widget, XtCallbackProc, XtPointer);	/* xmgobang.c */

extern void soundCB(Widget, XtPointer, XtPointer);	/* sounds.c */
extern void playSound(String);				/* sounds.c */
extern void kill_soundchild();				/* sounds.c */

extern void startPlayerCB(Widget, XtPointer, XtPointer);	/* options.c */
extern void playerTypeCB(Widget, XtPointer, XtPointer);	/* options.c */
extern void playModeCB(Widget, XtPointer, XtPointer);	/* options.c */
extern void cheatModeCB(Widget, XtPointer, XtPointer);	/* options.c */


extern void aboutCB(Widget, XtPointer, XtPointer);	/* help.c */
extern void helpCB(Widget, XtPointer, XtPointer);	/* help.c */


extern void undoCB(Widget, XtPointer, XtPointer);	/* gobang.c */
extern void redoCB(Widget, XtPointer, XtPointer);	/* gobang.c */
extern void pauseCB(Widget, XtPointer, XtPointer);	/* gobang.c */
extern void hintCB(Widget, XtPointer, XtPointer);	/* gobang.c */
extern void newGame();					/* gobang.c */
extern void allocBoard();				/* gobang.c */
extern void freeBoard();				/* gobang.c */
extern char getField(short, short);			/* gobang.c */
extern void setField(short, short, char);		/* gobang.c */
extern void makeMove(short, short);			/* gobang.c */


extern void createXImages();				/* xgfx.c */
extern void initGCs(Widget);				/* xgfx.c */
extern void drawBoard();				/* xgfx.c */
extern void displayGameOver(Boolean, short, short,
			    short, short, short);	/* xgfx.c */
extern void drawChipAt(int, int, short);		/* xgfx.c */
extern void undoChipAt(int, int);			/* xgfx.c */
extern void exposeCB(Widget, XtPointer, XtPointer);	/* xgfx.c */
extern void gfxinputCB(Widget, XtPointer, XtPointer);	/* xgfx.c */
extern void destroyAll();				/* xgfx.c */


extern void setCursor(short);				/* cursor.h */
extern void createCursor(Widget);			/* cursor.h */
extern void destroyCursors();				/* cursor.h */

/* global variables */

extern XtAppContext	context;		/* xmgobang.c */
extern Display 		*display;		/* xmgobang.c */
extern Widget		applShell;		/* xmgobang.c */

extern GoGame		XGoGame;		/* xmgobang.c */


